@@ -1,9 +1,14 @@
 #!/usr/bin/env python
-
+# encoding: utf8
 import wmldata, os, glob, sys
 import re
 
-"""Module implementing a WML parser."""
+"""
+NOTE: You should use wmlparser2.py instead which uses the C++
+preprocessor.
+
+Module implementing a WML parser in pure python.
+"""
 
 class Error(Exception):
     """
@@ -70,7 +75,7 @@ class Parser:
         self.macro_callback = None
         self.macro_not_found_callback = None
         self.no_macros = False
-        
+
         # If set to a function, the function is called with the current
         # textdomain and string as arguments for any translatable string, and
         # is expected to return a translation.
@@ -86,8 +91,6 @@ class Parser:
         # Whether to print current file, comments, and macro replacements.
         self.verbose = False
 
-        self.ignore_fatal_errors = False
-
     def read_encoded(self, filename):
         """
         Helper for gracefully handling non-utf8 files and fixing up non-unix
@@ -279,8 +282,8 @@ class Parser:
                         break
                 else:
                     continue
-            
-            
+
+
             quotation = line.find('"', line_start)
             while quotation >= 0:
                 in_string = True
@@ -289,7 +292,7 @@ class Parser:
                 line_start = string_end + 1
                 in_string = False
                 quotation = line.find('"', line_start)
-            
+
             if not in_string:
                 end = line.find(string, line_start)
                 if end >= 0:
@@ -298,6 +301,19 @@ class Parser:
             text += line
         return text
 
+    def read_lua(self):
+        """
+        Read a << .... >> string, return contents literally.
+        """
+        text = ""
+        while not self.at_end():
+            c = self.read_next()
+            if c == ">" and self.peek_next() == ">":
+                self.read_next() #get rid of trailing >
+                return text
+            text += c
+        raise Error(self, "Unexpected end of file")
+
     def skip_whitespace_inside_statement(self):
         self.read_while(" \t\r\n")
         if not self.at_end():
@@ -338,10 +354,10 @@ class Parser:
 
         preserve = macro
         macro = macro[:-1] # Get rid of final }
-        
+
         if self.macro_callback:
             if not self.macro_callback(macro): return None
-        
+
         # If the macro starts with ~, assume a file in userdata.
         if macro[0] == "~":
             if self.user_dir:
@@ -374,10 +390,14 @@ class Parser:
                     return None
             # If it is a directory, parse all cfg files within.
             if os.path.isdir(dirpath):
+                # Note: glob.glob will try to return unicode filenames
+                # if you pass it an unicode string - but to deal with
+                # non-unicode filenames as are allowed in linux we
+                # convert to a byte-string.
                 # Execute all WML files in the directory.
-                files = glob.glob(dirpath + "/*.cfg")
+                files = glob.glob(str(dirpath + "/*.cfg"))
                 # And also execute directories with a _main.cfg.
-                files += glob.glob(dirpath + "/*/_main.cfg")
+                files += glob.glob(str(dirpath + "/*/_main.cfg"))
                 files.sort()
                 mc = dirpath + "/_main.cfg"
                 fc = dirpath + "/_final.cfg"
@@ -450,7 +470,7 @@ class Parser:
                     # Ignore () within strings
                     if c == '"':
                         in_string = not in_string
-                        
+
                     if not in_string:
                         if c == "(":
                             balance += 1
@@ -473,16 +493,9 @@ class Parser:
             text = macro.text
             for i in range(len(macro.params)):
                 if 1 + i >= len(params):
-                    if self.ignore_fatal_errors:
-                        sys.stderr.write("***TRYING TO IGNORE FATAL ERROR ***\n")
-                        sys.stderr.write("Not enough parameters for macro %s. " % name +
-                            "%d given but %d needed %s.\n" % (len(params) - 1,
-                                len(macro.params), macro.params))
-                        params.append("")
-                    else:
-                        raise Error(self, "Not enough parameters for macro %s. " % name +
-                            "%d given but %d needed %s." % (len(params) - 1,
-                                len(macro.params), macro.params))
+                    raise Error(self, "Not enough parameters for macro %s. " % name +
+                        "%d given but %d needed %s." % (len(params) - 1,
+                            len(macro.params), macro.params))
                 rep = params[1 + i]
                 # Handle gettext replacement here, since inside the macro
                 # the textdomain will be wrong.
@@ -491,16 +504,13 @@ class Parser:
                     qe = rep.find('"', q + 1)
                     rep = self.gettext(self.textdomain, rep[q + 1:qe])
                     rep = '"' + rep + '"'
-                if self.verbose:                        
+                if self.verbose:
                     #s = "Replacing {%s} with %s\n" % (macro.params[i], rep)
                     ##sys.stderr.write(s.encode("utf8"))
                     pass
                 text = text.replace("{%s}" % macro.params[i], rep)
 
             if text:
-                # Hack: Ignore empty lines at beginning of a macro at least one
-                # addon breaks otherwise.
-                while len(text) >= 1 and text[0] == '\n': text = text[1:]
                 self.push_text(name, text, initial_textdomain = macro.textdomain)
             else:
                 pass # empty macro, nothing to do
@@ -508,7 +518,8 @@ class Parser:
             if self.macro_not_found_callback:
                 keep_macro = self.macro_not_found_callback(self, name, params)
                 if keep_macro: return keep_macro
-            sys.stderr.write("No macro %s.\n" % name.encode("utf8"))
+            if self.verbose:
+                sys.stderr.write("No macro %s.\n" % name.encode("utf8"))
             if self.verbose:
                 sys.stderr.write(" (%s:%d)\n" % (self.filename, self.line))
             return name
@@ -545,6 +556,7 @@ class Parser:
         spaces = ""
         filename = "(None)"
         line = -1
+        got_lua = False
         while 1:
             if c == "{":
                 keep_macro = self.parse_macro()
@@ -556,6 +568,13 @@ class Parser:
             elif c == "\n":
                 break
             elif c == "#":
+                # FIXME
+                # Assume something like:
+                # name="picture.png="+
+                # #textdomain blah
+                # "translateable name"
+                #
+                # For now, we ignore the textdomain...
                 self.read_until("\n")
                 break
             elif c == "+":
@@ -574,9 +593,12 @@ class Parser:
                         variables += [variable]
                         variable = ""
                     else:
-                        variable += c  
+                        variable += c
             else:
-                if c == '"':
+                if c == "<" and self.peek_next() == "<":
+                    self.read_next() #skip the rest of the opening
+                    value += self.read_lua()
+                elif c == '"':
                     # We want the textdomain at the beginning of the string,
                     # the end of the string may be outside a macro and already
                     # in another textdomain.
@@ -636,6 +658,8 @@ class Parser:
         while 1:
             self.skip_whitespace_and_newlines()
             if self.at_end():
+                if state:
+                    raise Error(self, "Tag stack non-empty (%s) at end of data" % state)
                 break
             c = self.read_next()
             if c == "#": # comment or preprocessor
@@ -649,7 +673,7 @@ class Parser:
                         if name: params += [name]
                         if sep == "\n": break
                         self.read_while(" ")
-                        
+
                     text = self.read_lines_until("#enddef")
                     if text == None:
                         raise Error(self, "#define without #enddef")
@@ -661,9 +685,13 @@ class Parser:
 
                 elif self.check_for("undef "):
                     self.read_until(" ")
-                    name = self.read_until(" \n")
+                    name = self.read_until("\n")
                     name = name.rstrip()
+                    if " " in name:
+                        if self.verbose: sys.stderr.write("Stray symbols in #undef %s\n" % name)
+                        name = name.split(" ")[0]
                     if name in self.macros: del self.macros[name]
+                    elif self.verbose: sys.stderr.write("undef'd macro '%s' did not exist\n" % name)
                 elif self.check_for("ifdef ") or self.check_for("ifndef"):
 
                     what = "#" + self.read_until(" ").rstrip()
@@ -718,7 +746,7 @@ class Parser:
                         raise Error(self, "#else without #ifdef")
 
                     self.read_until("\n")
-                    
+
                     # We seen an #else - that means we are at the end of a
                     # conditional preprocessor block which has executed. So
                     # we should now ignore everything up to the #endif.
@@ -757,22 +785,27 @@ class Parser:
                 if name[0] == '/':
                     if state == name[1:]:
                         return
-                    if self.ignore_fatal_errors:
-                        # This is just a hack to get broken addons to still show
-                        # up with wmlunits.
-                        sys.stderr.write("***TRYING TO IGNORE FATAL ERROR ***\n")
-                        sys.stderr.write("Mismatched closing tag [%s], expected [/%s]\n" % (name, state))
-                        continue
-                    else:
-                        raise Error(self, "Mismatched closing tag [%s], expected [/%s]" % (name, state))
-                #TODO: make a [+tag] properly append to most recent [tag]
-                # The below is an ugly hack, but better than keeping the '+' in the tag name.
+                    raise Error(self, "Mismatched closing tag [%s], expected [/%s]" % (name, state))
                 elif name[0] == '+':
                     name = name[1:]
-                subdata = wmldata.DataSub(name)
-                subdata.set_meta(self.filename, self.line)
-                self.parse_top(subdata, name)
-                data.insert(subdata)
+                    try:
+                        subdata = data.dict[name][-1]
+                        self.parse_top(subdata, name)
+                    except KeyError:
+                        subdata = wmldata.DataSub(name)
+                        subdata.set_meta(self.filename, self.line)
+                        self.parse_top(subdata, name)
+                        data.insert(subdata)
+                elif name[0] == " ":
+                    # We single this case out explicitly as the wesnoth parser
+                    # ignores it due to implementation reasons and this makes
+                    # more sense of the error message
+                    raise Error(self, "Invalid tag [%s] contains whitespace" % name)
+                else:
+                    subdata = wmldata.DataSub(name)
+                    subdata.set_meta(self.filename, self.line)
+                    self.parse_top(subdata, name)
+                    data.insert(subdata)
             elif c == '{':
                 keep_macro = self.parse_macro()
                 if isinstance(keep_macro, wmldata.Data):
@@ -782,12 +815,60 @@ class Parser:
                     data.insert(subdata)
 
 
+import json
+def jsonify(tree, verbose=False, depth=0):
+    """
+Convert a DataSub into JSON
+
+If verbose, insert a linebreak after every brace and comma (put every item on its own line), otherwise, condense everything into a single line.
+"""
+    print "{",
+    first = True
+    sdepth1 = "\n" + " " * depth
+    sdepth2 = sdepth1 + " "
+    for child in tree.children():
+        if first:
+            first = False
+        else:
+            sys.stdout.write(",")
+        if verbose:
+            sys.stdout.write(sdepth2)
+        print'"%s":' % child.name,
+        if child.get_type() == "DataSub":
+            jsonify(child, verbose, depth + 1)
+        else:
+            print json.dumps(child.get_value()),
+    if verbose:
+        sys.stdout.write(sdepth1)
+    sys.stdout.write("}")
+
+from xml.sax.saxutils import escape
+def xmlify(tree, verbose=False, depth=0):
+    sdepth = ""
+    if verbose:
+        sdepth = "  " * depth
+    for child in tree.children():
+        if child.get_type() == "DataSub":
+            print '%s<%s>' % (sdepth, child.name)
+            xmlify(child, verbose, depth + 1)
+            print '%s</%s>' % (sdepth, child.name)
+        else:
+            if "\n" in child.get_value() or "\r" in child.get_value():
+                print sdepth + '<' + child.name + '>' + \
+            '<![CDATA[' + child.get_value() + ']]>' + '</' + child.name + '>'
+            else:
+                print sdepth + '<' + child.name + '>' + \
+            escape(child.get_value())  + '</' + child.name + '>'
+
 if __name__ == "__main__":
     import optparse, subprocess
     try: import psyco
     except ImportError: pass
     else: psyco.full()
 
+    # Hack to make us not crash when we encounter characters that aren't ASCII
+    sys.stdout = __import__("codecs").getwriter('utf-8')(sys.stdout)
+
     optionparser = optparse.OptionParser()
     optionparser.set_usage("usage: %prog [options] [filename]")
     optionparser.add_option("-p", "--path",  help = "specify wesnoth data path")
@@ -801,6 +882,10 @@ if __name__ == "__main__":
         help = "do not expand any macros")
     optionparser.add_option("-c", "--contents", action = "store_true",
         help = "display contents of every tag")
+    optionparser.add_option("-j", "--to-json", action = "store_true",
+        help = "output JSON version of tree")
+    optionparser.add_option("-x", "--to-xml", action = "store_true",
+        help = "output XML version of tree")
     options, args = optionparser.parse_args()
 
     if options.path:
@@ -824,7 +909,7 @@ if __name__ == "__main__":
             print "gettext: '%s' '%s'" % (domain, x)
             return x
         wmlparser.gettext = gt
-    
+
     wmlparser.do_preprocessor_logic = True
 
     if options.execute:
@@ -837,5 +922,9 @@ if __name__ == "__main__":
     data = wmldata.DataSub("WML")
     wmlparser.parse_top(data)
 
-    data.debug(show_contents = options.contents, use_color = options.color)
-
+    if options.to_json:
+        jsonify(data, True) # For more readable results
+    elif options.to_xml:
+        xmlify(data, True)
+    else:
+        data.debug(show_contents = options.contents, use_color = options.color)
