@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2009 - 2012 by Bartosz Waresiak <dragonking@o2.pl>
+   Copyright (C) 2009 - 2013 by Bartosz Waresiak <dragonking@o2.pl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,10 +19,11 @@
 
 #include "ai.hpp"
 #include "candidates.hpp"
-#include "foreach.hpp"
 #include "../../log.hpp"
 #include "../../resources.hpp"
 
+#include <boost/foreach.hpp>
+
 static lg::log_domain log_formula_ai("ai/engine/fai");
 #define ERR_AI LOG_STREAM(err, log_formula_ai)
 
@@ -32,7 +33,7 @@ namespace game_logic {
 void candidate_action_manager::load_config(const config& cfg, ai::formula_ai* ai, function_symbol_table* function_table)
 {
 	// register candidate actions
-	foreach (const config &rc_action, cfg.child_range("register_candidate_action"))
+	BOOST_FOREACH(const config &rc_action, cfg.child_range("register_candidate_action"))
 	{
 		candidate_action_ptr new_ca = load_candidate_action_from_config(rc_action,ai,function_table);
 
@@ -67,7 +68,7 @@ bool candidate_action_manager::evaluate_candidate_actions(ai::formula_ai* ai, un
 {
 	evaluated_candidate_actions_.clear();
 
-	foreach(candidate_action_ptr cm, candidate_actions_)
+	BOOST_FOREACH(candidate_action_ptr cm, candidate_actions_)
 	{
 		cm->evaluate(ai, units);
 		evaluated_candidate_actions_.insert(cm);
@@ -116,7 +117,7 @@ candidate_action_with_filters::candidate_action_with_filters(
 	const config & filter_params = cfg.child("filter");
 
 	if( filter_params ) {
-		foreach( const config::attribute filter_param, filter_params.attribute_range() )
+		BOOST_FOREACH( const config::attribute filter_param, filter_params.attribute_range() )
 		{
 			game_logic::const_formula_ptr filter_formula(
 					new game_logic::formula(filter_param.second, function_table));
