@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2009 - 2012 by Bartosz Waresiak <dragonking@o2.pl>
+   Copyright (C) 2009 - 2013 by Bartosz Waresiak <dragonking@o2.pl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,6 @@
 
 #include "../../attack_prediction.hpp"
 #include "../../filesystem.hpp"
-#include "../../foreach.hpp"
 #include "../../game_display.hpp"
 #include "../../log.hpp"
 #include "../../map_label.hpp"
@@ -36,6 +35,7 @@
 #include "../../unit.hpp"
 #include "../../pathfind/pathfind.hpp"
 
+#include <boost/foreach.hpp>
 
 static lg::log_domain log_formula_ai("ai/engine/fai");
 #define LOG_AI LOG_STREAM(info, log_formula_ai)
@@ -355,7 +355,7 @@ private:
 				if( scores[current_side][i] > 98 )
 					continue;
 
-				foreach( int side , enemies) {
+				BOOST_FOREACH( int side , enemies) {
 					int diff = scores[current_side][i] - scores[side][i];
 					if ( diff > enemy_tollerancy) {
 						valid = false;
@@ -365,7 +365,7 @@ private:
 				}
 
 				if( valid ) {
-					foreach( int side , allies) {
+					BOOST_FOREACH( int side , allies) {
 						if ( scores[current_side][i] - scores[side][i] > ally_tollerancy ) {
 							valid = false;
 							break;
@@ -563,7 +563,7 @@ private:
                     visited_locs.erase(starting_loc);
 
                 std::vector<variant> res;
-                foreach( const map_location& ml, visited_locs) {
+                BOOST_FOREACH( const map_location& ml, visited_locs) {
                     res.push_back( variant(new location_callable( ml ) ) );
                 }
 
@@ -841,7 +841,7 @@ private:
 		ai::attack_analysis* analysis = convert_variant<ai::attack_analysis>(attack);
 		//unit_map units_with_moves(*resources::units);
 		//typedef std::pair<map_location, map_location> mv;
-		//foreach (const mv &m, analysis->movements) {
+		//BOOST_FOREACH(const mv &m, analysis->movements) {
 		//	units_with_moves.move(m.first, m.second);
 		//}
 
@@ -1629,7 +1629,7 @@ private:
 
 		std::vector<attack_type> attacks = attacker.attacks();
 
-		foreach(const attack_type &attack, attacks) {
+		BOOST_FOREACH(const attack_type &attack, attacks) {
 			const int dmg = round_damage(attack.damage(), defender.damage_from(attack), 100) * attack.num_attacks();
 			if (attack.range() == "melee") {
 				highest_melee_damage = std::max(highest_melee_damage, dmg);
