@@ -1,4 +1,4 @@
-/* $Id: campaign_server.cpp 55984 2013-01-01 09:34:55Z mordante $ */
+/* $Id$ */
 /*
    Copyright (C) 2003 - 2013 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -36,6 +36,7 @@
 
 #include <boost/foreach.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
+#include <boost/exception/get_error_info.hpp>
 
 // the fork execute is unix specific only tested on Linux quite sure it won't
 // work on Windows not sure which other platforms have a problem with it.
@@ -408,10 +409,16 @@ namespace {
 								break;
 							}
 						}
+
+						// TODO: remove for next add-ons server instance.
+						bool illegal_name_upload = false;
+
 						if (!data) {
 							LOG_CS << "Upload aborted - no add-on data.\n";
 							network::send_data(construct_error("Add-on rejected: No add-on data was supplied."), sock);
-						} else if (!addon_name_legal(upload["name"])) {
+						} else if ((illegal_name_upload = !addon_name_legal(upload["name"])) && campaign == NULL) {
+							// Only deny upload if we don't have an add-on with that id/name
+							// already. TODO: remove for next add-ons server instance.
 							LOG_CS << "Upload aborted - invalid add-on name.\n";
 							network::send_data(construct_error("Add-on rejected: The name of the add-on is invalid."), sock);
 						} else if (is_text_markup_char(upload["name"].str()[0])) {
@@ -486,6 +493,13 @@ namespace {
 								network::send_data(construct_error("Add-on rejected: The add-on already exists, and your passphrase was incorrect."), sock);
 							}
 						} else {
+							// Warn admins in the log about reuploading add-ons whose names don't
+							// pass the addon_name_legal() whitelist check above.
+
+							if(illegal_name_upload) {
+								LOG_CS << "Ignoring invalid add-on name '" << upload["name"] << "' because it already exists on the server.\n";
+							}
+
 							LOG_CS << "Upload is owner upload.\n";
 							std::string message = "Add-on accepted.";
 
@@ -611,9 +625,20 @@ namespace {
 					LOG_CS << "client disconnect: " << e.message << " " << network::ip_address(e.socket) << "\n";
 					e.disconnect();
 				}
-			} catch(config::error& /*e*/) {
-				LOG_CS << "error in receiving data...\n";
-				network::disconnect(sock);
+			} catch(const config::error& e) {
+				network::connection err_sock = 0;
+				network::connection const * err_connection = boost::get_error_info<network::connection_info>(e);
+				if(err_connection != NULL) {
+					err_sock = *err_connection;
+				}
+				if(err_sock == 0 && sock > 0)
+					err_sock = sock;
+				if(err_sock) {
+					LOG_CS << "client disconnect due to exception: " << e.what() << " " << network::ip_address(err_sock) << "\n";
+					network::disconnect(err_sock);
+				} else {
+					throw;
+				}
 			}
 
 			SDL_Delay(20);
