@@ -1,4 +1,4 @@
-/* $Id: display.cpp 55984 2013-01-01 09:34:55Z mordante $ */
+/* $Id$ */
 /*
    Copyright (C) 2003 - 2013 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -1723,7 +1723,7 @@ void display::scroll_to_tiles(const std::vector<map_location>& locs,
 	//if everything is fogged or the locs list is empty
 	if(!valid) return;
 
-	if (scroll_type == ONSCREEN || ONSCREEN_WARP) {
+	if (scroll_type == ONSCREEN || scroll_type == ONSCREEN_WARP) {
 		SDL_Rect r = map_area();
 		int spacing = round_double(add_spacing*hex_size());
 		r.x += spacing;
@@ -1746,7 +1746,7 @@ void display::scroll_to_tiles(const std::vector<map_location>& locs,
 	int target_x = locs_bbox.x + locs_bbox.w/2;
 	int target_y = locs_bbox.y + locs_bbox.h/2;
 
-	if (scroll_type == ONSCREEN || ONSCREEN_WARP) {
+	if (scroll_type == ONSCREEN || scroll_type == ONSCREEN_WARP) {
 		// when doing an ONSCREEN scroll we do not center the target unless needed
 		SDL_Rect r = map_area();
 		int map_center_x = r.x + r.w/2;
