@@ -1,4 +1,4 @@
-/* $Id: font.hpp 55984 2013-01-01 09:34:55Z mordante $ */
+/* $Id$ */
 /*
    Copyright (C) 2003 - 2013 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -215,16 +215,4 @@ void cache_mode(CACHE mode);
 
 }
 
-namespace {
-void function_to_hide_unused_variable_warning(){
-	(void) font::SIZE_TINY;
-	(void) font::SIZE_SMALL;
-
-	(void) font::SIZE_15;
-	(void) font::SIZE_PLUS;
-	(void) font::SIZE_LARGE;
-	(void) font::SIZE_XLARGE;
-}
-}
-
 #endif
