@@ -1,4 +1,4 @@
-/* $Id: image_modifications.cpp 55984 2013-01-01 09:34:55Z mordante $ */
+/* $Id$ */
 /*
    Copyright (C) 2009 - 2013 by Ignacio R. Morelle <shadowm2006@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -185,11 +185,11 @@ surface crop_modification::operator()(const surface& src) const
 		area.h = src->h;
 	}
 	if(area.x < 0) {
-		ERR_DP << "start X coordinate of SECTION modification is negative - truncating to zero\n";
+		ERR_DP << "start X coordinate of CROP modification is negative - truncating to zero\n";
 		area.x = 0;
 	}
 	if(area.y < 0) {
-		ERR_DP << "start Y coordinate of SECTION modification is negative - truncating to zero\n";
+		ERR_DP << "start Y coordinate of CROP modification is negative - truncating to zero\n";
 		area.y = 0;
 	}
 	return cut_surface(src, area);
