@@ -1,4 +1,4 @@
-/* $Id: network.hpp 55984 2013-01-01 09:34:55Z mordante $ */
+/* $Id$ */
 /*
    Copyright (C) 2003 - 2013 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -30,6 +30,7 @@ class config;
 #include <vector>
 
 #include <boost/shared_ptr.hpp>
+#include <boost/exception/error_info.hpp>
 
 namespace threading
 {
@@ -267,6 +268,9 @@ struct error : public game::error
 	void disconnect();
 };
 
+typedef boost::error_info<struct tag_tcpsocket,TCPsocket> tcpsocket_info;
+typedef boost::error_info<struct tag_connum,connection> connection_info;
+
 struct statistics
 {
 	statistics() : total(0), current(0), current_max(0) {}
