@@ -1,4 +1,4 @@
-/* $Id: tstring.hpp 55984 2013-01-01 09:34:55Z mordante $ */
+/* $Id$ */
 /*
    Copyright (C) 2004 - 2013 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -145,7 +145,7 @@ public:
 	static t_string from_serialized(const std::string& string) { return t_string(base::from_serialized(string)); }
 	std::string to_serialized() const { return get().to_serialized(); }
 
-	operator t_string_base() const { return get(); }
+	operator const t_string_base &() const { return get(); }
 
 	t_string operator+(const t_string& o) const { return get() + o.get(); }
 	t_string operator+(const std::string& o) const { return get() + o; }
